'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = packFeature;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('mz/fs');

var _fs2 = _interopRequireDefault(_fs);

var _writeFilePromise = require('write-file-promise');

var _writeFilePromise2 = _interopRequireDefault(_writeFilePromise);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _base = require('../lib/base62');

var _generateCode = require('../lib/generateCode');

var _generateCode2 = _interopRequireDefault(_generateCode);

var _getContents = require('../lib/getContents');

var _getContents2 = _interopRequireDefault(_getContents);

var _moduleExports = require('../lib/moduleExports');

var _moduleExports2 = _interopRequireDefault(_moduleExports);

var _invertObject = require('../lib/invertObject');

var _invertObject2 = _interopRequireDefault(_invertObject);

var _stringifyObject = require('../lib/stringifyObject');

var _stringifyObject2 = _interopRequireDefault(_stringifyObject);

var _statuses = require('../lib/statuses');

var _statuses2 = _interopRequireDefault(_statuses);

var _supported = require('../lib/supported');

var _supported2 = _interopRequireDefault(_supported);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var browsers = require('../../data/browsers');
var versions = require('../../data/browserVersions');

var browsersInverted = (0, _invertObject2.default)(browsers);
var statusesInverted = (0, _invertObject2.default)(_statuses2.default);
var versionsInverted = (0, _invertObject2.default)(versions);

var base = _path2.default.join(_path2.default.dirname(require.resolve('caniuse-db/data.json')), 'features-json');

var getContents = (0, _getContents2.default)(base);

function requireCall(path) {
    return t.callExpression(t.identifier('require'), [t.stringLiteral(path)]);
}

function featureIndex(data) {
    return (0, _generateCode2.default)(t.program([(0, _moduleExports2.default)(t.objectExpression(Object.keys(data).reduce(function (list, key) {
        var value = data[key];
        list.push(t.objectProperty(t.stringLiteral(key), requireCall(value)));
        return list;
    }, [])))]));
}

function packSupport(support) {
    var parts = support.split(' ');

    return parts.reduce(function (bitmask, part) {
        if (_supported2.default[part]) {
            return bitmask + _supported2.default[part];
        }
        // Handle notes - #1 = 128, #2 = 256, #3 = 512, etc
        return bitmask + Math.pow(2, parseInt(part.slice(1), 10) + 6);
    }, 0);
}

function packFeature() {
    return _fs2.default.readdir(base).then(getContents).then(function (features) {
        var index = features.reduce(function (map, feature) {
            var name = feature.name;

            map[name] = './features/' + name;
            return map;
        }, {});
        return (0, _writeFilePromise2.default)(_path2.default.join(__dirname, '../../data/features.js'), featureIndex(index)).then(function () {
            return features;
        });
    }).then(function (features) {
        return Promise.all(features.map(function (feature) {
            var name = feature.name,
                contents = feature.contents;

            var packed = {};
            packed.A = Object.keys(contents.stats).reduce(function (browserStats, key) {
                var browser = contents.stats[key];
                var supportData = Object.keys(browser).reduce(function (stats, version) {
                    var support = browser[version];
                    stats[versionsInverted[version]] = packSupport(support);
                    return stats;
                }, {});
                var compacted = Object.keys(supportData).reduce(function (min, k) {
                    var value = supportData[k];
                    if (!min[value]) {
                        min[value] = '' + k;
                    } else {
                        min[value] += ' ' + k;
                    }
                    return min;
                }, {});
                browserStats[browsersInverted[key]] = compacted;
                return browserStats;
            }, {});
            packed.B = parseInt(statusesInverted[contents.status], 10);
            packed.C = contents.title;
            return (0, _writeFilePromise2.default)(_path2.default.join(__dirname, '../../data/features/' + name + '.js'), (0, _stringifyObject2.default)(packed));
        }));
    });
}